-- Copyright (C) 2010 Noé Falzon

-- This program is  free software:  you can redistribute it and/or modify it
-- under  the terms  of the  GNU General Public License  as published by the
-- Free Software Foundation, version 3 of the License.
--
-- This program  is  distributed  in the hope  that it will  be useful,  but
-- WITHOUT  ANY WARRANTY;  without  even the implied  warranty of MERCHANTA-
-- BILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
-- License for more details.
--
-- You should have received a copy of the  GNU General Public License  along
-- with this program. If not, see <http://www.gnu.org/licenses/>.

require "class"
obj = require "objloader"
require "Menu"
require "Dude"

Boat = class()

local loadicon
local unloadicon

function Boat:init(x,y)
	
	self.x = x
	self.y = y
	
	self.z = game:altitude(x,y)
	
	self.gx = x
	self.gy = y
	
	self.load = 0
	self.dudepos = {}
	
	self.rad = 8
	
	if self.z > game.sealevel then
		return
	end
	
	if not loadicon then
		loadicon = obj.load("meshes/loadicon.obj")
		obj.makeDisplayList(loadicon, false, false)
	end
	
	if not unloadicon then
		unloadicon = obj.load("meshes/unloadicon.obj")
		obj.makeDisplayList(unloadicon, false, false)
	end
	
	return self
end


function Boat:update(dt)
	
	self.z = game.sealevel
	
	-- point to goal
	
	local dx = self.gx - self.x
	local dy = self.gy - self.y
	local dist = math.sqrt( dx^2 + dy^2 )
	
	if dist < 0.1 then
		dx,dy = 0,0
		dist = 1
	end
	
	
	-- move
	local speed = 1.5

	
	local x2 = self.x + dx/dist * speed * dt
	local y2 = self.y + dy/dist * speed * dt
	
	-- dont fall off
	
	if x2 > game.size-0.1 then x2 = game.size-0.1 end
	if y2 > game.size-0.1 then y2 = game.size-0.1 end
	if x2 < 0.1 then x2 = 0.1 end
	if y2 < 0.1 then y2 = 0.1 end
	
	-- don't go to the land
	if game:altitude(x2,y2) > game.sealevel then
		-- load/unload
		self:hitground()
		
		
		-- stop
		self.gx = self.x
		self.gy = self.y
	else
		
		self.x = x2
		self.y = y2
	end
	
end

function Boat:draw()
	
	if game.hot == self then
		gl.Color(col.hot)
	else
		gl.Color(col.buildings)
	end
		
	gl.PushMatrix()
	gl.Translate(self.x,self.y,self.z)
	gl.Scale(0.5,0.5,0.5)
	gl.Rotate(90,1,0,0)
	obj.drawDisplayList(boatmesh)
	gl.Rotate(-90,1,0,0)
	
	gl.Color(col.dudes)
	for i,v in ipairs(self.dudepos) do
		
		gl.PushMatrix()
		gl.Translate(v[1],v[2],0)
		gl.Scale(0.2,0.2,0.2)
		obj.drawDisplayList(dudemesh)
		gl.PopMatrix()
	end
	
	
	gl.PopMatrix()
	
	if self.loadtargeting or self.unloadtargeting then
		gl.Disable('LIGHTING')
		
		gl.LineWidth(5)
		gl.Color(col.hot)
		gl.Begin('LINES')
			gl.Vertex(self.x,self.y,self.z + 0.5)
			gl.Vertex(game.camx,game.camy,game.camalt + 0.1)
		gl.End('LINES')
		
		if self.loadtargeting then
			gl.LineWidth(3)
			gl.Begin('LINE_STRIP')
			local n = 60
			for i = 0,n do
				local x = game.camx + cos(i/n*2*math.pi)*self.rad
				local y = game.camy + sin(i/n*2*math.pi)*self.rad
				local z = game:altitude(x,y) + 2

				gl.Vertex(x,y,z)
			end
			gl.End('LINE_STRIP')
		end		
		
		gl.Enable('LIGHTING')
	end
	
end

function Boat:action()
	
	gMenu = Menu{
		{loadicon, function()
			
			game.targeting = self
			self.loadtargeting = true
			
		end},
		
		{unloadicon, function()
			
			game.targeting = self
			self.unloadtargeting = true
			
		end},
		
		{deletemesh, function()
			
			self.die = true
		end},
	}

end

function Boat:target()
	
	if self.loadtargeting then
		game.targeting = nil
		self.loadtargeting = false
		
		self.loading = true
		
		self.gx = game.camx
		self.gy = game.camy
		
		return
	end
	
	if self.unloadtargeting then
		game.targeting = nil
		self.unloadtargeting = false
		
		self.unloading = true
		
		self.gx = game.camx
		self.gy = game.camy
		
		return
	end
	
end

function Boat:canceltargeting()
	
	self.unloadtargeting = false
	self.loadtargeting = false
end

function Boat:hitground()
	
	if self.loading then
		for i,v in ipairs(game.dudes) do
			
			if dist2(self,v) < self.rad^2 then
				v:target(self.x,self.y)
			end
		end
		
		self.loading = false
		return
	end
	
	if self.unloading then
		
		-- find position
		local dx,dy = game.land:gradient(self.x,self.y)
		if game:altitude(self.x+dx, self.y+dy) > game.sealevel then
		
			for i = 1,self.load do
				local dud = Dude(game,1,self.x+dx,self.y+dy)
				TI(game.dudes, dud)
			end
			
			self.dudepos = {}
			self.load = 0
		end
		
		self.unloading = false
		return
	end
end

function Boat:embark()
	
	if self.load >= 10 then
		return	-- failure
	end
	
	self.load = self.load + 1
	TI(self.dudepos, {MR2(),MR2()})
		
	return true -- success
end
