-- Copyright (C) 2010 Noé Falzon

-- This program is  free software:  you can redistribute it and/or modify it
-- under  the terms  of the  GNU General Public License  as published by the
-- Free Software Foundation, version 3 of the License.
--
-- This program  is  distributed  in the hope  that it will  be useful,  but
-- WITHOUT  ANY WARRANTY;  without  even the implied  warranty of MERCHANTA-
-- BILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
-- License for more details.
--
-- You should have received a copy of the  GNU General Public License  along
-- with this program. If not, see <http://www.gnu.org/licenses/>.

require "class"

Heightfield = class()

function Heightfield:init(n)
	
	self.mesh = nil		-- 2D array of points
	
	self.faces = nil	-- userdata vArray
	self.normals = nil	-- userdata vArray
	self.numvert = nil	-- size of the two vArrays
	
	if n then
		self:genPerlinHeightmap(n)
		self.n = n
	end
	
	self:makeMesh()
	
	return self
end


function Heightfield:makeMesh()

		-- now make the list of tris
	local tris = {}
	local norms = {}
	
	local mesh = self.mesh
	
	for i = 0,#mesh-1 do
		for j = 0,#mesh[1]-1 do
			
			-- first half
			
			TI(tris,mesh[i][j])
			TI(tris,mesh[i+1][j])
			TI(tris,mesh[i][j+1])
			
			local v1 = mesh[i][j]
			local v2 = mesh[i+1][j]
			local v3 = mesh[i][j+1]
			
			local n = cross_prod_norm(v1,v2,v3)
				
			TI(norms,n)
			TI(norms,n)
			TI(norms,n)
			
			-- second half
			
			TI(tris,mesh[i+1][j])
			TI(tris,mesh[i+1][j+1])
			TI(tris,mesh[i][j+1])
			
			local v1 = mesh[i+1][j]
			local v2 = mesh[i+1][j+1]
			local v3 = mesh[i][j+1]
			
			local n = cross_prod_norm(v1,v2,v3)
				
			TI(norms,n)
			TI(norms,n)
			TI(norms,n)
			
		end
	end
	
	-- flatten tables of vertices
	local tris2 = {}
	for i,v in ipairs(tris) do
		TI(tris2,v[1])
		TI(tris2,v[2])
		TI(tris2,v[3])
	end
	
	local norms2 = {}
	for i,v in ipairs(norms) do
		TI(norms2,v[1])
		TI(norms2,v[2])
		TI(norms2,v[3])
	end

	-- make display list	

	self.dlist = gl.GenLists(1)
	gl.NewList(self.dlist, 'COMPILE')
	gl.Begin('TRIANGLES')
	for i,v in ipairs(tris) do
		gl.Normal(norms[i])
		gl.Vertex(v)
	end
	gl.End('TRIANGLES')
	gl.EndList()

end

function Heightfield:draw()
	
	gl.Color(col['land'..landcol])
	gl.Enable('POLYGON_OFFSET_FILL')
	gl.PolygonOffset(1,1)
	gl.CallList(self.dlist)
	gl.Disable('POLYGON_OFFSET_FILL')
	
	local t = col['land'..landcol]
	
	if wireframe then
		gl.Color(t[1]*0.9,t[2]*0.9,t[3]*0.9)
		gl.LineWidth(2)
		gl.PolygonMode('FRONT_AND_BACK','LINE')
		gl.CallList(self.dlist)
		gl.PolygonMode('FRONT_AND_BACK','FILL')
	end
	
end


function Heightfield:genPerlinHeightmap(n)

	local m = {}
	for i = 0,n do
		m[i] = {}
		
		for j = 0,n do
			m[i][j] = {i,j,0}
		end
	end
	
	local function noise(isln,height,width)
		-- seed a few high spots
		local mesh = {}
		
		for k=1,isln do
			local x,y = MR(n),MR(n)
			local h = MR()*0.5 + 0.5
			
			for i = 0,n do
				for j = 0,n do
					m[i][j][3] = m[i][j][3] + math.exp(-((x-i)^2+(y-j)^2)/width) * h
				end
			end
		end

		for i = 0,n do
			for j = 0,n do
				m[i][j][3] = math.sqrt(m[i][j][3]/isln) * height
			end
		end
		
	end
	
	noise(MF(n/5),60,80)
	
	-- noise
	local ampl = 30
	local wl = 40
	
	for k = 1,6 do
		
		seed_perlin(200)
		for i = 0,n do
			for j = 0,n do
				m[i][j][3] = m[i][j][3] + read_perlin(i,j,wl)*ampl-ampl/2
			end
		end
	
		ampl = ampl/2
		wl = wl/2
	end	
	
	self.mesh = m
end

function Heightfield:altitude(x,y)
	
	local i,j = MF(x),MF(y)
	local px,py = x-i,y-j
	local m = self.mesh
	
	
	if i >= self.n or j >= self.n or i < 0 or j < 0 then
		return
	end
	
	if px + py < 1 then
		local pxx = px/(1-py)
		local t = lerp(m[i][j][3],m[i+1][j][3], pxx)
		
		return lerp(t,m[i][j+1][3],py)
	else
		
		if py == 0 then
			return m[i+1][j][3]
		end
		
		local pxx = 1-(1-px)/py
		local t = lerp(m[i][j+1][3],m[i+1][j+1][3], pxx)
		
		return lerp(m[i+1][j][3],t,py)
	end
	
	return self.mesh[i][j][3]
end

-- just point at the highest corner of the cell

function Heightfield:gradient(x,y)
	
	local i,j = MF(x),MF(y)
	local px,py = x-i,y-j
	local m = self.mesh
	
	if i >= self.n or j >= self.n or i < 0 or j < 0 then
		return
	end
	
	local t = {m[i][j],m[i][j+1],m[i+1][j],m[i+1][j+1]}
	
	table.sort(t, function(a,b)
		return a[3] < b[3]
	end)
	
	local dx,dy = t[4][1] - x, t[4][2] - y
	local norm = math.sqrt(dx^2+dy^2)
	return dx/norm,dy/norm
end
