-- Copyright (C) 2010 Noé Falzon

-- This program is  free software:  you can redistribute it and/or modify it
-- under  the terms  of the  GNU General Public License  as published by the
-- Free Software Foundation, version 3 of the License.
--
-- This program  is  distributed  in the hope  that it will  be useful,  but
-- WITHOUT  ANY WARRANTY;  without  even the implied  warranty of MERCHANTA-
-- BILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
-- License for more details.
--
-- You should have received a copy of the  GNU General Public License  along
-- with this program. If not, see <http://www.gnu.org/licenses/>.

require "class"

local top = 2.298

local sheltercallmesh

Shelter = class()

function Shelter:init(x,y,z)
	
	self.x = x
	self.y = y
	self.z = z
	
	self.rad = 2.5
	
	self.load = 0
	
	-- check for water
	if z <= game.sealevel then
		return
	end
	
	self.building = 100	-- percents
	
	if not sheltercallmesh then
		sheltercallmesh = obj.load("meshes/sheltercall.obj")
		obj.makeDisplayList(sheltercallmesh, false, false)
	end
	
	return self
end

function Shelter:draw()
	
	if game.hot == self then
		gl.Color(col.hot)
	else
		gl.Color(col.buildings)
	end
	
	gl.Enable('LIGHTING')
	
	-- main mesh
	
	if self.building > 0 then
		gl.Enable('CLIP_PLANE0')
	end
	
	gl.PushMatrix()
	gl.Translate(self.x,self.y,self.z)
	gl.Rotate(90,1,0,0)
	
	gl.ClipPlane('CLIP_PLANE0',{0,-1,0,top*(100-self.building)/100})
	
	obj.drawDisplayList(sheltermesh)
	
	gl.Disable('CLIP_PLANE0')
	
	gl.PopMatrix()
	
	if game.hot == self then
	
		gl.LineWidth(3)
		gl.Begin('LINE_STRIP')
		local n = 60
		for i = 0,n do
			local x = self.x + cos(i/n*2*math.pi)*self.rad
			local y = self.y + sin(i/n*2*math.pi)*self.rad
			local z = game:altitude(x,y) + 1

			gl.Vertex(x,y,z)
		end
		gl.End('LINE_STRIP')
	end
	
end

function Shelter:update(dt)

	if self.z + 1.22 <= game.sealevel then
		
		if self.building > 0 then
			self.die = true
		else
		
			-- float!
			self.z = game.sealevel - 1.22
		end
	end
	
	-- build
	-- count people around
	local count = 0
	for i,v in ipairs(game.dudes) do
		if dist2(self,v) < self.rad^2 then
			count = count + 1
		end
	end
	
	if self.building > 0 then
		self.building = self.building - count/100*dt
	end
end

function Shelter:action()


	local cb = {}
	
	if self.building <= 0 then
	
		TI(cb,{sheltercallmesh, function()
					
			for i,v in ipairs(game.dudes) do
			
				if dist2(self,v) < self.rad^2 then
					v:target(self.x,self.y)
				end

			end
		end})
	end
	
	gMenu = Menu(cb)

end


function Shelter:embark(v)
	
	if self.load >= 100 then
		return	-- failure
	end
	
	self.load = self.load + 1
		
	return true -- success
end
